==> tinyca-0.4.3.tar.gz: TinyCA/lib/KEY.pm <==
# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: KEY.pm,v 1.2 2002/09/27 19:47:06 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package KEY;

sub new {
   my $self = {};
   my $that = shift;
   my $class = ref($that) || $that;

   bless($self, $class);
}

sub del_key {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'keytab'}->{'list'}->info('selection');

   my($keyname, $key, $keyfile, $action);

   if(not defined $ind) {
      $main->print_warning("No Key selected");
      return;
   }

   $keyname = $self->{'keylist'}->[$ind];

   $key = MIME::Base64::encode($keyname, '');

   $keyfile = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$key.".pem";

   if(not -s $keyfile) {
      $main->print_warning("Key not found");
      return;
   }

   $action = $main->show_key_del_confirm();

   return if($action eq 'Cancel');

   unlink($keyfile);

   $main->create_mframe();

   return;
}

sub read_keylist {
   my $self   = shift;
   my $main   = shift;
   my $ca     = $main->{'CA'}->{'actca'};
   my $keydir = $main->{'CA'}->{$ca}->{'dir'}."/keys";

   my ($f, $modt);

   my $keylist = [];

   $modt = (stat($keydir))[9];

   if(defined($self->{'lastread'}) &&
      $self->{'lastread'} >= $modt) { 
      return(0); 
   }

   opendir(DIR, $keydir) || do {
      $main->print_warning("can't open keydir");
      return(0);
   };

   while($f = readdir(DIR)) {
      next if $f =~ /^\./;
      $f =~ s/\.pem//;
      push(@{$keylist}, MIME::Base64::decode($f));
   }
   @{$keylist} = sort(@{$keylist});
   closedir(DIR);

   $self->{'keylist'} = $keylist;

   $self->{'lastread'} = time();
   return(1);  # got new list
}

sub export_key {
   my $self = shift;
   my $main = shift;
   my $ca  = $main->{'CA'}->{'actca'};
   my $ind = $main->{'keytab'}->{'list'}->info('selection');
    
   my($keyname, $key, $certfile, $outfile, $action, $format, 
      $x509, $out, $keyfile, $passwd, $p12passwd, $cafile, 
      $nopass, $includeca);

   if(not defined $ind) {
      $main->print_warning("No Key selected!");
      return;
   }

   $key = $self->{'keylist'}->[$ind];

   $keyname  = MIME::Base64::encode($key, '');
   $keyfile  = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$keyname.".pem";

   $nopass = 0;
   $action = $main->show_key_export_dialog( \$outfile, \$format, \$nopass);

   return if $action eq "Cancel";
   return if(not defined($outfile));

   if($nopass && $format eq 'P12') {
      $main->print_warning("Can't export PKCS#12 without passphrase");
      return;
   }

   if($nopass || $format eq 'DER') {
      $action = $main->show_key_nopasswd_dialog( \$passwd );
      return if $action eq "Cancel";
   }

   if(($format eq 'PEM') || ($format eq 'DER')) {
      unless(($format eq 'PEM') && not $nopass) {
         $out = $main->{'OpenSSL'}->convkey(
               'main'    => $main,
               'inform'  => 'PEM',
               'outform' => $format,
               'nopass'  => $nopass,
               'pass'    => $passwd,
               'keyfile' => $keyfile
               );
      }

      defined($out) || do {
         $main->print_warning("Converting failed");
         return;
      };

      open(OUT, ">$outfile") || do {
         $main->print_warning("Can't open Outfile!");
         return;
      };

      print OUT $out;
      close(OUT);

      $main->print_info("Key Exported to $outfile");
      return;

   } elsif ($format eq 'P12') {
      $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$keyname.".pem";
      $cafile   = $main->{'CA'}->{$ca}->{'dir'}."/cacert.pem";
      if(not -s $certfile) {
         $main->print_warning("Certificate is necessary for export as PKCS#12");
         return;
      }

      $includeca = 0;
      $action = $main->show_p12_export_dialog(
            \$p12passwd,
            \$passwd,
            \$includeca
            );

      return if $action eq "Cancel";
      
      unlink($outfile);
      $main->{'OpenSSL'}->genp12(
            main      => $main,
            certfile  => $certfile,
            keyfile   => $keyfile,
            cafile    => $cafile,
            outfile   => $outfile,
            passwd    => $passwd,
            p12passwd => $p12passwd,
            includeca => $includeca
            );

      if(not -s $outfile) {
         $main->print_warning("genp12() Failed");
         return;
      }

      $main->print_info("Certificate and Key exported to $outfile");
      return;

   } else {
      $main->print_warning("Invalid Format: $format");
      return;
   }

   $main->print_warning("Something Failed ??");

   return;
}

1

#
# $Log: KEY.pm,v $
# Revision 1.2  2002/09/27 19:47:06  sm
# Changed call to convkey() from open(...) to system(...)
#
#

==> tinyca-0.4.6.tar.gz: TinyCA/lib/KEY.pm <==
# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: KEY.pm,v 1.5 2003/06/23 20:11:30 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package KEY;

sub new {
   my $self = {};
   my $that = shift;
   my $class = ref($that) || $that;

   bless($self, $class);
}

sub del_key {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'keytab'}->{'list'}->info('selection');

   my($keyname, $key, $keyfile, $action);

   if(not defined $ind) {
      $main->print_info("Please select a key first");
      return;
   }

   $keyname = $self->{'keylist'}->[$ind];

   $key = MIME::Base64::encode($keyname, '');

   $keyfile = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$key.".pem";

   if(not -s $keyfile) {
      $main->print_warning("Key not found");
      return;
   }

   $action = $main->show_key_del_confirm();

   return if($action eq 'Cancel');

   unlink($keyfile);

   $main->create_mframe();

   return;
}

sub read_keylist {
   my $self   = shift;
   my $main   = shift;
   my $ca     = $main->{'CA'}->{'actca'};
   my $keydir = $main->{'CA'}->{$ca}->{'dir'}."/keys";

   my ($f, $modt, $tmp);

   my $keylist = [];

   $modt = (stat($keydir))[9];

   if(defined($self->{'lastread'}) &&
      $self->{'lastread'} >= $modt) { 
      return(0); 
   }

   opendir(DIR, $keydir) || do {
      $main->print_warning("can't open keydir");
      return(0);
   };

   while($f = readdir(DIR)) {
      next if $f =~ /^\./;
      $f =~ s/\.pem//;
      $tmp = MIME::Base64::decode($f);
      next if not defined($tmp);
      next if $tmp eq "";
      push(@{$keylist}, $tmp);
   }
   @{$keylist} = sort(@{$keylist});
   closedir(DIR);

   $self->{'keylist'} = $keylist;

   $self->{'lastread'} = time();
   return(1);  # got new list
}

sub export_key {
   my $self = shift;
   my $main = shift;
   my $ca  = $main->{'CA'}->{'actca'};
   my $ind = $main->{'keytab'}->{'list'}->info('selection');
    
   my($keyname, $key, $certfile, $outfile, $action, $format, 
      $x509, $out, $keyfile, $passwd, $p12passwd, $cafile, 
      $nopass, $includeca);

   if(not defined $ind) {
      $main->print_info("Please select a key first");
      return;
   }

   $key = $self->{'keylist'}->[$ind];

   $keyname  = MIME::Base64::encode($key, '');
   $keyfile  = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$keyname.".pem";

   $nopass = 0;
   $action = $main->show_key_export_dialog( \$outfile, \$format, \$nopass);

   return if $action eq "Cancel";
   return if(not defined($outfile));

   if($nopass && $format eq 'P12') {
      $main->print_warning("Can't export PKCS#12 without passphrase");
      return;
   }

   if($nopass || $format eq 'DER') {
      $action = $main->show_key_nopasswd_dialog( \$passwd );
      return if $action eq "Cancel";
   }

   if(($format eq 'PEM') || ($format eq 'DER')) {
      unless(($format eq 'PEM') && not $nopass) {
         $out = $main->{'OpenSSL'}->convkey(
               'main'    => $main,
               'inform'  => 'PEM',
               'outform' => $format,
               'nopass'  => $nopass,
               'pass'    => $passwd,
               'keyfile' => $keyfile
               );

         defined($out) || do {
            $main->print_warning("Converting failed");
            return;
         };
      }

      if(($format eq 'PEM') && not $nopass) {
         open(IN, "<$keyfile") || do {
            $main->print_warning("Can't open Keyfile: \'$keyfile\' $!");
            return;
         };
         $out .= $_ while(<IN>);
         close(IN);
      }

      open(OUT, ">$outfile") || do {
         $main->print_warning("Can't open Outfile: \'$outfile\' $!");
         return;
      };

      print OUT $out;
      close(OUT);

      $main->print_info("Key Exported to $outfile");
      return;

   } elsif ($format eq 'P12') {
      $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$keyname.".pem";
      $cafile   = $main->{'CA'}->{$ca}->{'dir'}."/cacert.pem";
      if(not -s $certfile) {
         $main->print_warning("Certificate is necessary for export as PKCS#12");
         return;
      }

      $includeca = 0;
      $action = $main->show_p12_export_dialog(
            \$p12passwd,
            \$passwd,
            \$includeca
            );

      return if $action eq "Cancel";
      
      unlink($outfile);
      $main->{'OpenSSL'}->genp12(
            main      => $main,
            certfile  => $certfile,
            keyfile   => $keyfile,
            cafile    => $cafile,
            outfile   => $outfile,
            passwd    => $passwd,
            p12passwd => $p12passwd,
            includeca => $includeca
            );

      if(not -s $outfile) {
         $main->print_warning("genp12() Failed");
         return;
      }

      $main->print_info("Certificate and Key exported to $outfile");
      return;

   } else {
      $main->print_warning("Invalid Format: $format");
      return;
   }

   $main->print_warning("Something Failed ??");

   return;
}

1

#
# $Log: KEY.pm,v $
# Revision 1.5  2003/06/23 20:11:30  sm
# some new texts from ludwig.nussel@suse.de
#
# Revision 1.4  2002/10/04 09:21:04  sm
# skip empty lines when decoding failed
#
# Revision 1.3  2002/10/04 08:46:45  sm
# fixed bug exporting keys in PEM format
#
# Revision 1.2  2002/09/27 19:47:06  sm
# Changed call to convkey() from open(...) to system(...)
#
#
